import os
from pathlib import Path

os.environ.setdefault("BOT_TOKEN", "TEST_TOKEN")

import bot


def test_wx_effective_config_prefers_env(monkeypatch, tmp_path):
    monkeypatch.setenv("VIBEGO_CONFIG_DIR", str(tmp_path))
    monkeypatch.setenv("WX_APPID", "ENV_APPID")
    monkeypatch.setenv("WX_PKP", "/abs/path/from/env/private.key")
    monkeypatch.setenv("PROJECT_PATH", "/abs/project")
    monkeypatch.setattr(bot, "PROJECT_NAME", "demo-wx")

    env_file = tmp_path / "wx_ci" / "demo-wx.env"
    env_file.parent.mkdir(parents=True, exist_ok=True)
    env_file.write_text("WX_APPID=FILE_APPID\nWX_PKP=/from/file.key\nPROJECT_PATH=/from/file\n", encoding="utf-8")

    conf = bot._wx_effective_config()
    assert conf["appid"] == "ENV_APPID"
    assert conf["pkp_path"] == "/abs/path/from/env/private.key"
    assert conf["project_path"] == "/abs/project"
    assert conf["env_file"].endswith("demo-wx.env")


def test_wx_missing_reasons(monkeypatch, tmp_path):
    monkeypatch.setenv("VIBEGO_CONFIG_DIR", str(tmp_path))
    conf = {"appid": "", "pkp_path": "", "project_path": ""}
    reasons = bot._wx_missing_reasons(conf)
    assert any("WX_APPID" in item for item in reasons)
    assert any("WX_PKP" in item for item in reasons)

    missing_file_conf = {"appid": "appid", "pkp_path": "/non/exist.key", "project_path": ""}
    reasons = bot._wx_missing_reasons(missing_file_conf)
    assert any("/non/exist.key" in item for item in reasons)


def test_write_wx_env_file(monkeypatch, tmp_path):
    monkeypatch.setenv("VIBEGO_CONFIG_DIR", str(tmp_path))
    monkeypatch.setattr(bot, "PROJECT_NAME", "demo-wx")
    env_path = bot._write_wx_env_file("appid123", "/abs/key.key", "/abs/project")

    assert env_path.exists()
    content = env_path.read_text(encoding="utf-8")
    assert "WX_APPID=appid123" in content
    assert "WX_PKP=/abs/key.key" in content
    assert "PROJECT_PATH=/abs/project" in content
    mode = env_path.stat().st_mode & 0o777
    assert mode in {0o600, 0o644, 0o666}  # 允许 umask 影响，但确保可读写



import logging
from optparse import Values
from typing import Any

from pip._vendor.packaging.markers import default_environment
from pip._vendor.rich import print_json

from pip import __version__
from pip._internal.cli import cmdoptions
from pip._internal.cli.base_command import Command
from pip._internal.cli.status_codes import SUCCESS
from pip._internal.metadata import BaseDistribution, get_environment
from pip._internal.utils.compat import stdlib_pkgs
from pip._internal.utils.urls import path_to_url

logger = logging.getLogger(__name__)


class InspectCommand(Command):
    """
    Inspect the content of a Python environment and produce a report in JSON format.
    """

    ignore_require_venv = True
    usage = """
      %prog [options]"""

    def add_options(self) -> None:
        self.cmd_opts.add_option(
            "--local",
            action="store_true",
            default=False,
            help=(
                "If in a virtualenv that has global access, do not list "
                "globally-installed packages."
            ),
        )
        self.cmd_opts.add_option(
            "--user",
            dest="user",
            action="store_true",
            default=False,
            help="Only output packages installed in user-site.",
        )
        self.cmd_opts.add_option(cmdoptions.list_path())
        self.parser.insert_option_group(0, self.cmd_opts)

    def run(self, options: Values, args: list[str]) -> int:
        cmdoptions.check_list_path_option(options)
        dists = get_environment(options.path).iter_installed_distributions(
            local_only=options.local,
            user_only=options.user,
            skip=set(stdlib_pkgs),
        )
        output = {
            "version": "1",
            "pip_version": __version__,
            "installed": [self._dist_to_dict(dist) for dist in dists],
            "environment": default_environment(),
            # TODO tags? scheme?
        }
        print_json(data=output)
        return SUCCESS

    def _dist_to_dict(self, dist: BaseDistribution) -> dict[str, Any]:
        res: dict[str, Any] = {
            "metadata": dist.metadata_dict,
            "metadata_location": dist.info_location,
        }
        # direct_url. Note that we don't have download_info (as in the installation
        # report) since it is not recorded in installed metadata.
        direct_url = dist.direct_url
        if direct_url is not None:
            res["direct_url"] = direct_url.to_dict()
        else:
            # Emulate direct_url for legacy editable installs.
            editable_project_location = dist.editable_project_location
            if editable_project_location is not None:
                res["direct_url"] = {
                    "url": path_to_url(editable_project_location),
                    "dir_info": {
                        "editable": True,
                    },
                }
        # installer
        installer = dist.installer
        if dist.installer:
            res["installer"] = installer
        # requested
        if dist.installed_with_dist_info:
            res["requested"] = dist.requested
        return res



"""aiogram FSM 状态定义。"""
from __future__ import annotations

from aiogram.fsm.state import State, StatesGroup


class TaskCreateStates(StatesGroup):
    """任务创建引导流程的状态集。"""

    waiting_title = State()
    waiting_type = State()
    waiting_description = State()
    waiting_confirm = State()


class TaskEditStates(StatesGroup):
    """任务编辑流程的状态集。"""

    waiting_task_id = State()
    waiting_field_choice = State()
    waiting_new_value = State()
    waiting_confirm = State()


class TaskNoteStates(StatesGroup):
    """任务备注追加流程的状态集。"""

    waiting_task_id = State()
    waiting_content = State()
    waiting_type = State()


class TaskBugReportStates(StatesGroup):
    """缺陷报告流程状态。"""

    waiting_description = State()
    waiting_reproduction = State()
    waiting_logs = State()
    waiting_confirm = State()


class TaskDescriptionStates(StatesGroup):
    """任务描述编辑流程状态。"""

    waiting_content = State()
    waiting_confirm = State()


class TaskPushStates(StatesGroup):
    """任务推送补充信息流程的状态集。"""

    waiting_choice = State()
    waiting_supplement = State()


class TaskAttachmentStates(StatesGroup):
    """任务附件绑定流程状态。"""

    waiting_files = State()


class TaskListSearchStates(StatesGroup):
    """任务列表搜索流程的状态集。"""

    waiting_keyword = State()


class ProjectDeleteStates(StatesGroup):
    """Master 项目删除确认流程的状态定义。"""

    confirming = State()


class WxConfigStates(StatesGroup):
    """微信小程序 CI 配置引导流程的状态集。"""

    waiting_appid = State()
    waiting_pkp = State()
    waiting_project_path = State()
    waiting_confirm = State()
